@@ -26,11 +26,13 @@
 #pragma once
 
 #include "functiontraits.h"
+#include "optional.h"
 #include "utils_global.h"
 
 #include <QCoreApplication>
 #include <QFuture>
 #include <QFutureInterface>
+#include <QFutureWatcher>
 #include <QRunnable>
 #include <QThread>
 #include <QThreadPool>
@@ -52,10 +54,13 @@ static testCallOperatorNo testCallOperator(...);
 template<typename T>
 struct hasCallOperator
 {
-    static const bool value = (sizeof(testCallOperator<T>(0)) == sizeof(testCallOperatorYes));
+    static const bool value = (sizeof(testCallOperator<T>(nullptr)) == sizeof(testCallOperatorYes));
 };
 
 namespace Utils {
+
+using StackSizeInBytes = optional<uint>;
+
 namespace Internal {
 
 /*
@@ -139,7 +144,7 @@ struct resultTypeIsFunctionLike<Function, false>
 
 template <typename Function>
 struct resultTypeHasCallOperator<Function, false>
-        : public resultTypeIsFunctionLike<Function, std::is_function<typename std::remove_pointer<typename std::decay<Function>::type>::type>::value>
+        : public resultTypeIsFunctionLike<Function, std::is_function<std::remove_pointer_t<std::decay_t<Function>>>::value>
 {
 };
 
@@ -233,39 +238,38 @@ private:
 
 // void function that does not take QFutureInterface
 template <typename ResultType, typename Function, typename... Args>
-void runAsyncReturnVoidDispatch(std::true_type, QFutureInterface<ResultType>, Function &&function, Args&&... args)
+void runAsyncReturnVoidDispatch(std::true_type, QFutureInterface<ResultType> &, Function &&function, Args&&... args)
 {
     function(std::forward<Args>(args)...);
 }
 
 // non-void function that does not take QFutureInterface
 template <typename ResultType, typename Function, typename... Args>
-void runAsyncReturnVoidDispatch(std::false_type, QFutureInterface<ResultType> futureInterface, Function &&function, Args&&... args)
+void runAsyncReturnVoidDispatch(std::false_type, QFutureInterface<ResultType> &futureInterface, Function &&function, Args&&... args)
 {
     futureInterface.reportResult(function(std::forward<Args>(args)...));
 }
 
 // function that takes QFutureInterface
 template <typename ResultType, typename Function, typename... Args>
-void runAsyncQFutureInterfaceDispatch(std::true_type, QFutureInterface<ResultType> futureInterface, Function &&function, Args&&... args)
+void runAsyncQFutureInterfaceDispatch(std::true_type, QFutureInterface<ResultType> &futureInterface, Function &&function, Args&&... args)
 {
     function(futureInterface, std::forward<Args>(args)...);
 }
 
 // function that does not take QFutureInterface
 template <typename ResultType, typename Function, typename... Args>
-void runAsyncQFutureInterfaceDispatch(std::false_type, QFutureInterface<ResultType> futureInterface, Function &&function, Args&&... args)
+void runAsyncQFutureInterfaceDispatch(std::false_type, QFutureInterface<ResultType> &futureInterface, Function &&function, Args&&... args)
 {
-    runAsyncReturnVoidDispatch(std::is_void<typename std::result_of<Function(Args...)>::type>(),
+    runAsyncReturnVoidDispatch(std::is_void<std::invoke_result_t<Function, Args...>>(),
                                futureInterface, std::forward<Function>(function), std::forward<Args>(args)...);
 }
 
 // function, function pointer, or other callable object that is no member pointer
 template <typename ResultType, typename Function, typename... Args,
-          typename = typename std::enable_if<
-                !std::is_member_pointer<typename std::decay<Function>::type>::value
-              >::type>
-void runAsyncMemberDispatch(QFutureInterface<ResultType> futureInterface, Function &&function, Args&&... args)
+          typename = std::enable_if_t<!std::is_member_pointer<std::decay_t<Function>>::value>
+         >
+void runAsyncMemberDispatch(QFutureInterface<ResultType> &futureInterface, Function &&function, Args&&... args)
 {
     runAsyncQFutureInterfaceDispatch(functionTakesArgument<Function, 0, QFutureInterface<ResultType>&>(),
                                      futureInterface, std::forward<Function>(function), std::forward<Args>(args)...);
@@ -273,20 +277,19 @@ void runAsyncMemberDispatch(QFutureInterface<ResultType> futureInterface, Functi
 
 // Function = member function
 template <typename ResultType, typename Function, typename Obj, typename... Args,
-          typename = typename std::enable_if<
-                std::is_member_pointer<typename std::decay<Function>::type>::value
-              >::type>
-void runAsyncMemberDispatch(QFutureInterface<ResultType> futureInterface, Function &&function, Obj &&obj, Args&&... args)
+          typename = std::enable_if_t<std::is_member_pointer<std::decay_t<Function>>::value>
+         >
+void runAsyncMemberDispatch(QFutureInterface<ResultType> &futureInterface, Function &&function, Obj &&obj, Args&&... args)
 {
     // Wrap member function with object into callable
     runAsyncImpl(futureInterface,
-                 MemberCallable<typename std::decay<Function>::type>(std::forward<Function>(function), std::forward<Obj>(obj)),
+                 MemberCallable<std::decay_t<Function>>(std::forward<Function>(function), std::forward<Obj>(obj)),
                  std::forward<Args>(args)...);
 }
 
 // cref to function/callable
 template <typename ResultType, typename Function, typename... Args>
-void runAsyncImpl(QFutureInterface<ResultType> futureInterface,
+void runAsyncImpl(QFutureInterface<ResultType> &futureInterface,
                   std::reference_wrapper<Function> functionWrapper, Args&&... args)
 {
     runAsyncMemberDispatch(futureInterface, functionWrapper.get(), std::forward<Args>(args)...);
@@ -294,7 +297,7 @@ void runAsyncImpl(QFutureInterface<ResultType> futureInterface,
 
 // function/callable, no cref
 template <typename ResultType, typename Function, typename... Args>
-void runAsyncImpl(QFutureInterface<ResultType> futureInterface,
+void runAsyncImpl(QFutureInterface<ResultType> &futureInterface,
                   Function &&function, Args&&... args)
 {
     runAsyncMemberDispatch(futureInterface, std::forward<Function>(function),
@@ -305,16 +308,8 @@ void runAsyncImpl(QFutureInterface<ResultType> futureInterface,
    arguments that are passed to it when it is run in a thread.
 */
 
-// can be replaced with std::(make_)index_sequence with C++14
-template <std::size_t...>
-struct indexSequence { };
-template <std::size_t N, std::size_t... S>
-struct makeIndexSequence : makeIndexSequence<N-1, N-1, S...> { };
-template <std::size_t... S>
-struct makeIndexSequence<0, S...> { typedef indexSequence<S...> type; };
-
 template <class T>
-typename std::decay<T>::type
+std::decay_t<T>
 decayCopy(T&& v)
 {
     return std::forward<T>(v);
@@ -334,7 +329,7 @@ public:
         futureInterface.reportStarted();
     }
 
-    ~AsyncJob()
+    ~AsyncJob() override
     {
         // QThreadPool can delete runnables even if they were never run (e.g. QThreadPool::clear).
         // Since we reported them as started, we make sure that we always report them as finished.
@@ -354,7 +349,7 @@ public:
             futureInterface.reportFinished();
             return;
         }
-        runHelper(typename makeIndexSequence<std::tuple_size<Data>::value>::type());
+        runHelper(std::make_index_sequence<std::tuple_size<Data>::value>());
     }
 
     void setThreadPool(QThreadPool *pool)
@@ -368,10 +363,10 @@ public:
     }
 
 private:
-    using Data = std::tuple<typename std::decay<Function>::type, typename std::decay<Args>::type...>;
+    using Data = std::tuple<std::decay_t<Function>, std::decay_t<Args>...>;
 
     template <std::size_t... index>
-    void runHelper(indexSequence<index...>)
+    void runHelper(std::index_sequence<index...>)
     {
         // invalidates data, which is moved into the call
         runAsyncImpl(futureInterface, std::move(std::get<index>(data))...);
@@ -388,15 +383,43 @@ private:
 class QTCREATOR_UTILS_EXPORT RunnableThread : public QThread
 {
 public:
-    explicit RunnableThread(QRunnable *runnable, QObject *parent = 0);
+    explicit RunnableThread(QRunnable *runnable, QObject *parent = nullptr);
 
 protected:
-    void run();
+    void run() override;
 
 private:
     QRunnable *m_runnable;
 };
 
+template<typename Function,
+         typename... Args,
+         typename ResultType = typename Internal::resultType<Function>::type>
+QFuture<ResultType> runAsync_internal(QThreadPool *pool,
+                                      StackSizeInBytes stackSize,
+                                      QThread::Priority priority,
+                                      Function &&function,
+                                      Args &&... args)
+{
+    Q_ASSERT(!(pool && stackSize)); // stack size cannot be changed once a thread is started
+    auto job = new Internal::AsyncJob<ResultType,Function,Args...>
+            (std::forward<Function>(function), std::forward<Args>(args)...);
+    job->setThreadPriority(priority);
+    QFuture<ResultType> future = job->future();
+    if (pool) {
+        job->setThreadPool(pool);
+        pool->start(job);
+    } else {
+        auto thread = new Internal::RunnableThread(job);
+        if (stackSize)
+            thread->setStackSize(stackSize.value());
+        thread->moveToThread(qApp->thread()); // make sure thread gets deleteLater on main thread
+        QObject::connect(thread, &QThread::finished, thread, &QObject::deleteLater);
+        thread->start(priority);
+    }
+    return future;
+}
+
 } // Internal
 
 /*!
@@ -427,20 +450,11 @@ template <typename Function, typename... Args,
 QFuture<ResultType>
 runAsync(QThreadPool *pool, QThread::Priority priority, Function &&function, Args&&... args)
 {
-    auto job = new Internal::AsyncJob<ResultType,Function,Args...>
-            (std::forward<Function>(function), std::forward<Args>(args)...);
-    job->setThreadPriority(priority);
-    QFuture<ResultType> future = job->future();
-    if (pool) {
-        job->setThreadPool(pool);
-        pool->start(job);
-    } else {
-        auto thread = new Internal::RunnableThread(job);
-        thread->moveToThread(qApp->thread()); // make sure thread gets deleteLater on main thread
-        QObject::connect(thread, &QThread::finished, thread, &QObject::deleteLater);
-        thread->start(priority);
-    }
-    return future;
+    return Internal::runAsync_internal(pool,
+                                       StackSizeInBytes(),
+                                       priority,
+                                       std::forward<Function>(function),
+                                       std::forward<Args>(args)...);
 }
 
 /*!
@@ -457,16 +471,57 @@ runAsync(QThread::Priority priority, Function &&function, Args&&... args)
                     std::forward<Function>(function), std::forward<Args>(args)...);
 }
 
+/*!
+    Runs \a function with \a args in a new thread with given thread \a stackSize and
+    thread priority QThread::InheritPriority .
+    \sa runAsync(QThreadPool*,QThread::Priority,Function&&,Args&&...)
+    \sa QThread::Priority
+    \sa QThread::setStackSize
+*/
+template<typename Function,
+         typename... Args,
+         typename ResultType = typename Internal::resultType<Function>::type>
+QFuture<ResultType> runAsync(Utils::StackSizeInBytes stackSize, Function &&function, Args &&... args)
+{
+    return Internal::runAsync_internal(static_cast<QThreadPool *>(nullptr),
+                                       stackSize,
+                                       QThread::InheritPriority,
+                                       std::forward<Function>(function),
+                                       std::forward<Args>(args)...);
+}
+
+/*!
+    Runs \a function with \a args in a new thread with given thread \a stackSize and
+    given thread \a priority.
+    \sa runAsync(QThreadPool*,QThread::Priority,Function&&,Args&&...)
+    \sa QThread::Priority
+    \sa QThread::setStackSize
+*/
+template<typename Function,
+         typename... Args,
+         typename ResultType = typename Internal::resultType<Function>::type>
+QFuture<ResultType> runAsync(Utils::StackSizeInBytes stackSize,
+                             QThread::Priority priority,
+                             Function &&function,
+                             Args &&... args)
+{
+    return Internal::runAsync_internal(static_cast<QThreadPool *>(nullptr),
+                                       stackSize,
+                                       priority,
+                                       std::forward<Function>(function),
+                                       std::forward<Args>(args)...);
+}
+
 /*!
     Runs \a function with \a args in a new thread with thread priority QThread::InheritPriority.
     \sa runAsync(QThreadPool*,QThread::Priority,Function&&,Args&&...)
     \sa QThread::Priority
  */
 template <typename Function, typename... Args,
-          typename = typename std::enable_if<
-                !std::is_same<typename std::decay<Function>::type, QThreadPool>::value
-                && !std::is_same<typename std::decay<Function>::type, QThread::Priority>::value
-              >::type,
+          typename = std::enable_if_t<
+                !std::is_same<std::decay_t<Function>, QThreadPool>::value
+                && !std::is_same<std::decay_t<Function>, QThread::Priority>::value
+              >,
           typename ResultType = typename Internal::resultType<Function>::type>
 QFuture<ResultType>
 runAsync(Function &&function, Args&&... args)
@@ -482,9 +537,7 @@ runAsync(Function &&function, Args&&... args)
     \sa QThread::Priority
  */
 template <typename Function, typename... Args,
-          typename = typename std::enable_if<
-                !std::is_same<typename std::decay<Function>::type, QThread::Priority>::value
-              >::type,
+          typename = std::enable_if_t<!std::is_same<std::decay_t<Function>, QThread::Priority>::value>,
           typename ResultType = typename Internal::resultType<Function>::type>
 QFuture<ResultType>
 runAsync(QThreadPool *pool, Function &&function, Args&&... args)
@@ -493,4 +546,113 @@ runAsync(QThreadPool *pool, Function &&function, Args&&... args)
                     std::forward<Args>(args)...);
 }
 
-} // Utils
+
+/*!
+    Adds a handler for when a result is ready.
+    This creates a new QFutureWatcher. Do not use if you intend to react on multiple conditions
+    or create a QFutureWatcher already for other reasons.
+*/
+template <typename R, typename T>
+const QFuture<T> &onResultReady(const QFuture<T> &future, R *receiver, void(R::*member)(const T &))
+{
+    auto watcher = new QFutureWatcher<T>();
+    QObject::connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    QObject::connect(watcher, &QFutureWatcherBase::resultReadyAt, receiver,
+                     [receiver, member, watcher](int index) {
+                         (receiver->*member)(watcher->future().resultAt(index));
+                     });
+    watcher->setFuture(future);
+    return future;
+}
+
+/*!
+    Adds a handler for when a result is ready. The guard object determines the lifetime of
+    the connection.
+    This creates a new QFutureWatcher. Do not use if you intend to react on multiple conditions
+    or create a QFutureWatcher already for other reasons.
+*/
+template <typename T, typename Function>
+const QFuture<T> &onResultReady(const QFuture<T> &future, QObject *guard, Function f)
+{
+    auto watcher = new QFutureWatcher<T>();
+    QObject::connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    QObject::connect(watcher, &QFutureWatcherBase::resultReadyAt, guard, [f, watcher](int index) {
+        f(watcher->future().resultAt(index));
+    });
+    watcher->setFuture(future);
+    return future;
+}
+
+/*!
+    Adds a handler for when a result is ready.
+    This creates a new QFutureWatcher. Do not use if you intend to react on multiple conditions
+    or create a QFutureWatcher already for other reasons.
+*/
+template <typename T, typename Function>
+const QFuture<T> &onResultReady(const QFuture<T> &future, Function f)
+{
+    auto watcher = new QFutureWatcher<T>();
+    QObject::connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    QObject::connect(watcher, &QFutureWatcherBase::resultReadyAt, [f, watcher](int index) {
+        f(watcher->future().resultAt(index));
+    });
+    watcher->setFuture(future);
+    return future;
+}
+
+/*!
+    Adds a handler for when the future is finished.
+    This creates a new QFutureWatcher. Do not use if you intend to react on multiple conditions
+    or create a QFutureWatcher already for other reasons.
+*/
+template<typename R, typename T>
+const QFuture<T> &onFinished(const QFuture<T> &future,
+                             R *receiver,
+                             void (R::*member)(const QFuture<T> &))
+{
+    auto watcher = new QFutureWatcher<T>();
+    QObject::connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    QObject::connect(watcher,
+                     &QFutureWatcherBase::finished,
+                     receiver,
+                     [receiver, member, watcher]() { (receiver->*member)(watcher->future()); });
+    watcher->setFuture(future);
+    return future;
+}
+
+/*!
+    Adds a handler for when the future is finished. The guard object determines the lifetime of
+    the connection.
+    This creates a new QFutureWatcher. Do not use if you intend to react on multiple conditions
+    or create a QFutureWatcher already for other reasons.
+*/
+template<typename T, typename Function>
+const QFuture<T> &onFinished(const QFuture<T> &future, QObject *guard, Function f)
+{
+    auto watcher = new QFutureWatcher<T>();
+    QObject::connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    QObject::connect(watcher, &QFutureWatcherBase::finished, guard, [f, watcher]() {
+        f(watcher->future());
+    });
+    watcher->setFuture(future);
+    return future;
+}
+
+/*!
+    Adds a handler for when the future is finished.
+    This creates a new QFutureWatcher. Do not use if you intend to react on multiple conditions
+    or create a QFutureWatcher already for other reasons.
+*/
+template<typename T, typename Function>
+const QFuture<T> &onFinished(const QFuture<T> &future, Function f)
+{
+    auto watcher = new QFutureWatcher<T>();
+    QObject::connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    QObject::connect(watcher, &QFutureWatcherBase::finished, [f, watcher]() {
+        f(watcher->future());
+    });
+    watcher->setFuture(future);
+    return future;
+}
+
+} // namespace Utils
