@@ -35,12 +35,6 @@
 #include "shortcutmanager.h"
 #include <designeractionmanager.h>
 
-#include <QStringList>
-
-QT_BEGIN_NAMESPACE
-class QAction;
-QT_END_NAMESPACE
-
 namespace Core {
     class IEditor;
 }
@@ -48,22 +42,22 @@ namespace Core {
 namespace QmlDesigner {
 
 class QmlDesignerPluginPrivate;
+class AsynchronousImageCache;
 
 namespace Internal { class DesignModeWidget; }
 
-class QMLDESIGNERCORE_EXPORT QmlDesignerPlugin : public ExtensionSystem::IPlugin
+class QMLDESIGNERCORE_EXPORT QmlDesignerPlugin final : public ExtensionSystem::IPlugin
 {
     Q_OBJECT
     Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QtCreatorPlugin" FILE "QmlDesigner.json")
 
 public:
     QmlDesignerPlugin();
-    virtual ~QmlDesignerPlugin();
+    ~QmlDesignerPlugin() final;
 
-    //Plugin
-    bool initialize(const QStringList &arguments, QString *errorMessage = 0) override;
-    bool delayedInitialize() override;
-    void extensionsInitialized() override;
+    bool initialize(const QStringList &arguments, QString *errorMessage) final;
+    bool delayedInitialize() final;
+    void extensionsInitialized() final;
 
     static QmlDesignerPlugin *instance();
 
@@ -89,6 +83,17 @@ public:
 
     static double formEditorDevicePixelRatio();
 
+    static void emitUsageStatistics(const QString &identifier);
+    static void emitUsageStatisticsContextAction(const QString &identifier);
+    static void emitUsageStatisticsTime(const QString &identifier, int elapsed);
+
+    static AsynchronousImageCache &imageCache();
+
+signals:
+    void usageStatisticsNotifier(const QString &identifier);
+    void usageStatisticsUsageTimer(const QString &identifier, int elapsed);
+
+
 private: // functions
     void integrateIntoQtCreator(QWidget *modeWidget);
     void showDesigner();
@@ -105,7 +110,6 @@ private: // functions
 private: // variables
     QmlDesignerPluginPrivate *d = nullptr;
     static QmlDesignerPlugin *m_instance;
-
 };
 
 } // namespace QmlDesigner
